/* Show the cost of using GUIDs as clustered keys */
use [tempdb]
GO
SET NOCOUNT ON;

/* Basic setup. */
IF EXISTS(SELECT * FROM sys.tables WHERE name = 'tblGUID')
BEGIN
	DROP TABLE dbo.tblGUID;
END

CREATE TABLE dbo.tblGUID
(
	Cluster uniqueidentifier NOT NULL,
	Filler char(250)
);

ALTER TABLE dbo.tblGUID ADD CONSTRAINT
	[PK_tblGUID] PRIMARY KEY(Cluster);

IF EXISTS(SELECT * FROM sys.tables WHERE name = 'tblIdentity')
BEGIN
	DROP TABLE dbo.tblIdentity;
END

CREATE TABLE dbo.tblIdentity
(
	Cluster int identity(1,1) NOT NULL,
	Filler char(250)
);

ALTER TABLE dbo.tblIdentity ADD CONSTRAINT
	[PK_tblIdentity] PRIMARY KEY(Cluster);

/* Time insertion of 1 million rows into each table. */
DECLARE 
	@GUIDStartTime DATETIME2(4),
	@GUIDEndTime DATETIME2(4),
	@IdentityStartTime DATETIME2(4),
	@IdentityEndTime DATETIME2(4);

SET @GUIDStartTime = CURRENT_TIMESTAMP;

INSERT INTO dbo.tblGUID
(
	Cluster,
	Filler
)
SELECT TOP(1000000)
	NEWID(),
	REPLICATE('A', 250)
FROM
	sys.objects o
	CROSS JOIN sys.objects o2
	CROSS JOIN sys.objects o3
	CROSS JOIN sys.objects o4;

SET @GUIDEndTime = CURRENT_TIMESTAMP;

SET @IdentityStartTime = CURRENT_TIMESTAMP;

INSERT INTO dbo.tblIdentity
(
	Filler
)
SELECT TOP(1000000)
	REPLICATE('A', 250)
FROM
	sys.objects o
	CROSS JOIN sys.objects o2
	CROSS JOIN sys.objects o3
	CROSS JOIN sys.objects o4;

SET @IdentityEndTime = CURRENT_TIMESTAMP;

SELECT
	DATEDIFF(MILLISECOND, @GuidStartTime, @GuidEndTime) as GUIDTime,
	DATEDIFF(MILLISECOND, @IdentityStartTime, @IdentityEndTime) AS IdentityTime;

/* Show table size and fragmentation level for each table. */
exec sp_spaceused 'tblGUID';
exec sp_spaceused 'tblIdentity';

SELECT 
	OBJECT_NAME(OBJECT_ID), 
	index_id,
	index_type_desc,
	index_level,
	avg_fragmentation_in_percent,
	avg_page_space_used_in_percent,
	page_count
FROM 
	sys.dm_db_index_physical_stats
	(
		DB_ID(N'tempdb'), 
		NULL, 
		NULL, 
		NULL, 
		'SAMPLED'
	)
ORDER BY 
	avg_fragmentation_in_percent DESC;






/* Looks OK so far, but let's throw in another "batch" of insertions. */

INSERT INTO dbo.tblGUID
(
	Cluster,
	Filler
)
SELECT TOP(50000)
	NEWID(),
	REPLICATE('A', 250)
FROM
	sys.objects o
	CROSS JOIN sys.objects o2
	CROSS JOIN sys.objects o3;

INSERT INTO dbo.tblIdentity
(
	Filler
)
SELECT TOP(50000)
	REPLICATE('A', 250)
FROM
	sys.objects o
	CROSS JOIN sys.objects o2
	CROSS JOIN sys.objects o3;

/* Show table size and fragmentation level for each table. */
exec sp_spaceused 'tblGUID';
exec sp_spaceused 'tblIdentity';

SELECT 
	OBJECT_NAME(OBJECT_ID), 
	index_id,
	index_type_desc,
	index_level,
	avg_fragmentation_in_percent,
	avg_page_space_used_in_percent,
	page_count
FROM 
	sys.dm_db_index_physical_stats
	(
		DB_ID(N'tempdb'), 
		NULL, 
		NULL, 
		NULL, 
		'SAMPLED'
	)
ORDER BY 
	avg_fragmentation_in_percent DESC;